@@ -30,12 +30,18 @@ import operator
 import math
 import inspect
 import types
+import itertools
 
 
 INF = float('inf')
 NAN = float('nan')
 NUMBER = numbers.Number
 BASESTRING_OR_NUMBER = NUMBER, basestring
+try:
+	import numpy as np
+	SEQUENCE = collections.Sequence, np.ndarray
+except ImportError:
+	SEQUENCE = collections.Sequence
 
 
 class BaseColumn(OrderedState):
@@ -101,7 +107,7 @@ class BaseColumn(OrderedState):
 
 		desc:
 			Arithmetic mean of all values. If there are non-numeric values,
-			these are ignored. If there are no numeric values, None or np.nan is
+			these are ignored. If there are no numeric values, NAN is
 			returned.
 		"""
 
@@ -118,7 +124,7 @@ class BaseColumn(OrderedState):
 
 		desc:
 			The median of all values. If there are non-numeric values,
-			these are ignored. If there are no numeric values, None or np.nan is
+			these are ignored. If there are no numeric values, NAN is
 			returned.
 		"""
 
@@ -138,8 +144,8 @@ class BaseColumn(OrderedState):
 
 		desc:
 			The standard deviation of all values. If there are non-numeric
-			values, these are ignored. If there are 0 or 1 numeric values, None
-			or np.nan is returned. The degrees of freedom are N-1.
+			values, these are ignored. If there are 0 or 1 numeric values, NAN
+			is returned. The degrees of freedom are N-1.
 		"""
 
 		m = self.mean
@@ -155,7 +161,7 @@ class BaseColumn(OrderedState):
 		name:	max
 
 		desc:
-			The highest numeric value in the column, or None or np.nan if there
+			The highest numeric value in the column, or NAN if there
 			are no numeric values.
 		"""
 
@@ -171,7 +177,7 @@ class BaseColumn(OrderedState):
 		name:	min
 
 		desc:
-			The lowest numeric value in the column, or None or np.nan if there
+			The lowest numeric value in the column, or NAN if there
 			are no numeric values.
 		"""
 
@@ -187,7 +193,7 @@ class BaseColumn(OrderedState):
 		name:	sum
 
 		desc:
-			The sum of all values in the column, or None or np.nan if there
+			The sum of all values in the column, or NAN if there
 			are no numeric values.
 		"""
 
@@ -350,12 +356,14 @@ class BaseColumn(OrderedState):
 
 		col = self._empty_col()
 		col._rowid = Index(_rowid)
-		col._seq = []
-		for row in _rowid:
-			if row in self._rowid:
-				col._seq.append(self._seq[self._rowid.index(row)])
-			else:
-				col._seq.append(other._seq[other._rowid.index(row)])
+		col._seq = [None] * len(_rowid)
+		self_row_id = set(self._rowid)
+		for i, row in enumerate(_rowid):
+			col._seq[i] = (
+				self._seq[self._rowid.index(row)]
+				if row in self_row_id
+				else other._seq[other._rowid.index(row)]
+			)
 		return col
 
 	def _tosequence(self, value, length=None):
@@ -380,15 +388,21 @@ class BaseColumn(OrderedState):
 
 		if length is None:
 			length = len(self._datamatrix)
-		if value is None or isinstance(value, (numbers.Number, basestring)):
-			return [self._checktype(value)]*length
+		if value is None or isinstance(value, BASESTRING_OR_NUMBER):
+			return [self._checktype(value)] * length
 		try:
-			value = list(value)
-		except:
-			raise Exception('Cannot convert to sequence: %s' % value)
-		if len(value) != length:
-			raise Exception('Sequence has incorrect length: %s' % len(value))
-		return [self._checktype(cell) for cell in value]
+			iter(value)
+		except TypeError:
+			raise TypeError('Cannot convert to sequence: %s' % value)
+		seq = [
+			self._checktype(cell)
+			for cell in itertools.islice(value, 0, length + 1)
+		]
+		if len(seq) != length:
+			raise ValueError(
+				'Sequence length does not match DataMatrix'
+			)
+		return seq
 
 	def _getintkey(self, key):
 
@@ -450,6 +464,22 @@ class BaseColumn(OrderedState):
 			col._seq.append(self._seq[i])
 		return col
 
+	def _getdatamatrixkey(self, key):
+
+		"""
+		visible: False
+
+		desc:
+			Gets a slice of this column by a DataMatrix
+
+		arguments:
+			key:	A DataMatrix
+		"""
+
+		if key != self._datamatrix:
+			raise ValueError('Cannot slice column with a different DataMatrix')
+		return self[[self._rowid.index(_rowid) for _rowid in key._rowid]]
+
 	def _getrowidkey(self, key):
 
 		"""
@@ -713,23 +743,18 @@ class BaseColumn(OrderedState):
 
 		col = self._empty_col()
 		col._rowid = self._rowid.copy()
-		col._seq = []
 		for i, (_other, val) in enumerate(
+			zip(self._seq, self._tosequence(other, len(self)))
+			if flip else
 			zip(self._tosequence(other, len(self)), self._seq)
 		):
-			if flip:
-				_other, val = val, _other
-			if (
-				isinstance(val, numbers.Number)
-				and isinstance(_other, numbers.Number)
-			):
-				col._seq.append(number_op(val, _other))
-			elif str_op is not None:
-				col._seq.append(
-					str_op(safe_decode(val), safe_decode(_other))
-				)
-			else:
-				col._seq.append(val)
+			if (isinstance(val, NUMBER) and isinstance(_other, NUMBER)):
+				col._seq[i] = number_op(val, _other)
+				continue
+			if str_op is not None:
+				col._seq[i] = str_op(safe_decode(val), safe_decode(_other))
+				continue
+			col._seq[i] = _other if flip else val
 		return col
 
 	def _map(self, fnc):
@@ -811,9 +836,11 @@ class BaseColumn(OrderedState):
 			return self._getintkey(key)
 		if isinstance(key, slice):
 			return self._getslicekey(key)
-		if isinstance(key, collections.Sequence):
+		if isinstance(key, SEQUENCE):
 			return self._getsequencekey(key)
-		raise Exception(u'Invalid key')
+		if isinstance(key, DataMatrix):
+			return self._getdatamatrixkey(key)
+		raise Exception(u'Invalid key: {}'.format(key))
 
 	def __setitem__(self, key, value):
 
@@ -821,14 +848,19 @@ class BaseColumn(OrderedState):
 			self._setintkey(key, value)
 		elif isinstance(key, slice):
 			self._setslicekey(key, value)
-		elif isinstance(key, collections.Sequence):
+		elif isinstance(key, SEQUENCE):
 			self._setsequencekey(key, value)
 		elif isinstance(key, DataMatrix):
 			self._setdatamatrixkey(key, value)
 		else:
-			raise Exception('Invalid assignment')
+			raise Exception('Invalid assignment: {} to {}'.format(key, value))
 		self._datamatrix._mutate()
 
+	def __array__(self, *args):
+
+		import numpy as np
+		return np.array(self._seq)
+
 	def __gt__(self, other):
 		return self._compare(other, operator.gt)
 	def __ge__(self, other):
